@@ -20,6 +20,7 @@ import (
 	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/tools/record"
+	"k8s.io/utils/pointer"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
@@ -68,6 +69,38 @@ func TestHasMatchingLabels(t *testing.T) {
 			},
 			expected: false,
 		},
+		{
+			machine: machine,
+			machineHealthCheck: &mapiv1beta1.MachineHealthCheck{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "EmptySelector",
+					Namespace: namespace,
+				},
+				TypeMeta: metav1.TypeMeta{
+					Kind: "MachineHealthCheck",
+				},
+				Spec: mapiv1beta1.MachineHealthCheckSpec{
+					Selector: metav1.LabelSelector{},
+				},
+				Status: mapiv1beta1.MachineHealthCheckStatus{},
+			},
+			expected: true,
+		},
+		{
+			machine: machine,
+			machineHealthCheck: &mapiv1beta1.MachineHealthCheck{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "NilSelector", // Note this shouldn't happen, API schema validation requires the selector be non-nil
+					Namespace: namespace,
+				},
+				TypeMeta: metav1.TypeMeta{
+					Kind: "MachineHealthCheck",
+				},
+				Spec:   mapiv1beta1.MachineHealthCheckSpec{},
+				Status: mapiv1beta1.MachineHealthCheckStatus{},
+			},
+			expected: true,
+		},
 	}
 
 	for _, tc := range testsCases {
@@ -196,6 +229,8 @@ func TestReconcile(t *testing.T) {
 	machineWithoutNodeRef.Status.NodeRef = nil
 
 	machineHealthCheck := maotesting.NewMachineHealthCheck("machineHealthCheck")
+	nodeStartupTimeout := 15 * time.Minute
+	machineHealthCheck.Spec.NodeStartupTimeout = nodeStartupTimeout.String()
 
 	// remediationExternal
 	nodeUnhealthyForTooLong := maotesting.NewNode("nodeUnhealthyForTooLong", false)
@@ -280,7 +315,7 @@ func TestReconcile(t *testing.T) {
 			node:     nodeAnnotatedWithMachineWithoutNodeReference,
 			expected: expectedReconcile{
 				result: reconcile.Result{
-					RequeueAfter: timeoutForMachineToHaveNode,
+					RequeueAfter: nodeStartupTimeout,
 				},
 				error: false,
 			},
@@ -330,11 +365,27 @@ func TestReconcile(t *testing.T) {
 	}
 }
 
-func TestHasMachineSetOwner(t *testing.T) {
+func TestHasControllerOwner(t *testing.T) {
 	machineWithMachineSet := maotesting.NewMachine("machineWithMachineSet", "node")
+
 	machineWithNoMachineSet := maotesting.NewMachine("machineWithNoMachineSet", "node")
 	machineWithNoMachineSet.OwnerReferences = nil
 
+	machineWithAnyControllerOwner := maotesting.NewMachine("machineWithAnyControllerOwner", "node")
+	machineWithAnyControllerOwner.OwnerReferences = []metav1.OwnerReference{
+		{
+			Kind:       "Any",
+			Controller: pointer.BoolPtr(true),
+		},
+	}
+
+	machineWithNoControllerOwner := maotesting.NewMachine("machineWithNoControllerOwner", "node")
+	machineWithNoControllerOwner.OwnerReferences = []metav1.OwnerReference{
+		{
+			Kind: "Any",
+		},
+	}
+
 	testsCases := []struct {
 		target   target
 		expected bool
@@ -351,10 +402,22 @@ func TestHasMachineSetOwner(t *testing.T) {
 			},
 			expected: true,
 		},
+		{
+			target: target{
+				Machine: *machineWithAnyControllerOwner,
+			},
+			expected: true,
+		},
+		{
+			target: target{
+				Machine: *machineWithNoControllerOwner,
+			},
+			expected: false,
+		},
 	}
 
 	for _, tc := range testsCases {
-		if got := tc.target.hasMachineSetOwner(); got != tc.expected {
+		if got := tc.target.hasControllerOwner(); got != tc.expected {
 			t.Errorf("Test case: Machine %s. Expected: %t, got: %t", tc.target.Machine.Name, tc.expected, got)
 		}
 	}
@@ -934,7 +997,8 @@ func TestGetTargetsFromMHC(t *testing.T) {
 							Labels: map[string]string{},
 						},
 						TypeMeta: metav1.TypeMeta{
-							Kind: "Node",
+							Kind:       "Node",
+							APIVersion: "v1",
 						},
 						Status: corev1.NodeStatus{
 							Conditions: []corev1.NodeCondition{},
@@ -961,7 +1025,8 @@ func TestGetTargetsFromMHC(t *testing.T) {
 						Labels: map[string]string{},
 					},
 					TypeMeta: metav1.TypeMeta{
-						Kind: "Node",
+						Kind:       "Node",
+						APIVersion: "v1",
 					},
 					Status: corev1.NodeStatus{
 						Conditions: []corev1.NodeCondition{},
@@ -977,7 +1042,8 @@ func TestGetTargetsFromMHC(t *testing.T) {
 						Labels: map[string]string{},
 					},
 					TypeMeta: metav1.TypeMeta{
-						Kind: "Node",
+						Kind:       "Node",
+						APIVersion: "v1",
 					},
 					Status: corev1.NodeStatus{
 						Conditions: []corev1.NodeCondition{},
@@ -998,7 +1064,8 @@ func TestGetTargetsFromMHC(t *testing.T) {
 							Labels: map[string]string{},
 						},
 						TypeMeta: metav1.TypeMeta{
-							Kind: "Node",
+							Kind:       "Node",
+							APIVersion: "v1",
 						},
 						Status: corev1.NodeStatus{
 							Conditions: []corev1.NodeCondition{},
@@ -1018,7 +1085,8 @@ func TestGetTargetsFromMHC(t *testing.T) {
 							Labels: map[string]string{},
 						},
 						TypeMeta: metav1.TypeMeta{
-							Kind: "Node",
+							Kind:       "Node",
+							APIVersion: "v1",
 						},
 						Status: corev1.NodeStatus{
 							Conditions: []corev1.NodeCondition{},
@@ -1098,7 +1166,7 @@ func TestGetTargetsFromMHC(t *testing.T) {
 		t.Run(tc.testCase, func(t *testing.T) {
 			got, err := newFakeReconciler(objects...).getTargetsFromMHC(*tc.mhc)
 			if !equality.Semantic.DeepEqual(got, tc.expectedTargets) {
-				t.Errorf("Case: %v. Got: %v, expected: %v", tc.testCase, got, tc.expectedTargets)
+				t.Errorf("Case: %v. Got: %+v, expected: %+v", tc.testCase, got, tc.expectedTargets)
 			}
 			if tc.expectedError != (err != nil) {
 				t.Errorf("Case: %v. Got: %v, expected error: %v", tc.testCase, err, tc.expectedError)
@@ -1160,7 +1228,8 @@ func TestGetNodeFromMachine(t *testing.T) {
 					Labels: map[string]string{},
 				},
 				TypeMeta: metav1.TypeMeta{
-					Kind: "Node",
+					Kind:       "Node",
+					APIVersion: "v1",
 				},
 				Status: corev1.NodeStatus{
 					Conditions: []corev1.NodeCondition{},
@@ -1244,11 +1313,12 @@ func TestNeedsRemediation(t *testing.T) {
 	knownDate := metav1.Time{Time: time.Date(1985, 06, 03, 0, 0, 0, 0, time.Local)}
 	machineFailed := machinePhaseFailed
 	testCases := []struct {
-		testCase                 string
-		target                   *target
-		expectedNeedsRemediation bool
-		expectedNextCheck        time.Duration
-		expectedError            bool
+		testCase                    string
+		target                      *target
+		timeoutForMachineToHaveNode time.Duration
+		expectedNeedsRemediation    bool
+		expectedNextCheck           time.Duration
+		expectedError               bool
 	}{
 		{
 			testCase: "healthy: does not met conditions criteria",
@@ -1307,9 +1377,10 @@ func TestNeedsRemediation(t *testing.T) {
 					Status: mapiv1beta1.MachineHealthCheckStatus{},
 				},
 			},
-			expectedNeedsRemediation: false,
-			expectedNextCheck:        time.Duration(0),
-			expectedError:            false,
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			expectedNeedsRemediation:    false,
+			expectedNextCheck:           time.Duration(0),
+			expectedError:               false,
 		},
 		{
 			testCase: "unhealthy: node does not exist",
@@ -1346,9 +1417,10 @@ func TestNeedsRemediation(t *testing.T) {
 					Status: mapiv1beta1.MachineHealthCheckStatus{},
 				},
 			},
-			expectedNeedsRemediation: true,
-			expectedNextCheck:        time.Duration(0),
-			expectedError:            false,
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			expectedNeedsRemediation:    true,
+			expectedNextCheck:           time.Duration(0),
+			expectedError:               false,
 		},
 		{
 			testCase: "unhealthy: nodeRef nil longer than timeout",
@@ -1364,7 +1436,7 @@ func TestNeedsRemediation(t *testing.T) {
 					},
 					Spec: mapiv1beta1.MachineSpec{},
 					Status: mapiv1beta1.MachineStatus{
-						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-timeoutForMachineToHaveNode) - 1*time.Second)},
+						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-defaultNodeStartupTimeout) - 1*time.Second)},
 					},
 				},
 				Node: nil,
@@ -1398,9 +1470,10 @@ func TestNeedsRemediation(t *testing.T) {
 					Status: mapiv1beta1.MachineHealthCheckStatus{},
 				},
 			},
-			expectedNeedsRemediation: true,
-			expectedNextCheck:        time.Duration(0),
-			expectedError:            false,
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			expectedNeedsRemediation:    true,
+			expectedNextCheck:           time.Duration(0),
+			expectedError:               false,
 		},
 		{
 			testCase: "unhealthy: meet conditions criteria",
@@ -1416,7 +1489,7 @@ func TestNeedsRemediation(t *testing.T) {
 					},
 					Spec: mapiv1beta1.MachineSpec{},
 					Status: mapiv1beta1.MachineStatus{
-						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-timeoutForMachineToHaveNode) - 1*time.Second)},
+						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-defaultNodeStartupTimeout) - 1*time.Second)},
 					},
 				},
 				Node: &corev1.Node{
@@ -1472,9 +1545,10 @@ func TestNeedsRemediation(t *testing.T) {
 					Status: mapiv1beta1.MachineHealthCheckStatus{},
 				},
 			},
-			expectedNeedsRemediation: true,
-			expectedNextCheck:        time.Duration(0),
-			expectedError:            false,
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			expectedNeedsRemediation:    true,
+			expectedNextCheck:           time.Duration(0),
+			expectedError:               false,
 		},
 		{
 			testCase: "unhealthy: machine phase failed",
@@ -1524,9 +1598,10 @@ func TestNeedsRemediation(t *testing.T) {
 					Status: mapiv1beta1.MachineHealthCheckStatus{},
 				},
 			},
-			expectedNeedsRemediation: true,
-			expectedNextCheck:        time.Duration(0),
-			expectedError:            false,
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			expectedNeedsRemediation:    true,
+			expectedNextCheck:           time.Duration(0),
+			expectedError:               false,
 		},
 		{
 			testCase: "healthy: meet conditions criteria but timeout",
@@ -1542,7 +1617,7 @@ func TestNeedsRemediation(t *testing.T) {
 					},
 					Spec: mapiv1beta1.MachineSpec{},
 					Status: mapiv1beta1.MachineStatus{
-						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-timeoutForMachineToHaveNode) - 1*time.Second)},
+						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-defaultNodeStartupTimeout) - 1*time.Second)},
 					},
 				},
 				Node: &corev1.Node{
@@ -1598,9 +1673,10 @@ func TestNeedsRemediation(t *testing.T) {
 					Status: mapiv1beta1.MachineHealthCheckStatus{},
 				},
 			},
-			expectedNeedsRemediation: false,
-			expectedNextCheck:        time.Duration(1 * time.Minute), // 300-200 rounded
-			expectedError:            false,
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			expectedNeedsRemediation:    false,
+			expectedNextCheck:           time.Duration(1 * time.Minute), // 300-200 rounded
+			expectedError:               false,
 		},
 		{
 			testCase: "error bad conditions timeout",
@@ -1616,7 +1692,7 @@ func TestNeedsRemediation(t *testing.T) {
 					},
 					Spec: mapiv1beta1.MachineSpec{},
 					Status: mapiv1beta1.MachineStatus{
-						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-timeoutForMachineToHaveNode) - 1*time.Second)},
+						LastUpdated: &metav1.Time{Time: time.Now().Add(time.Duration(-defaultNodeStartupTimeout) - 1*time.Second)},
 					},
 				},
 				Node: &corev1.Node{
@@ -1672,15 +1748,16 @@ func TestNeedsRemediation(t *testing.T) {
 					Status: mapiv1beta1.MachineHealthCheckStatus{},
 				},
 			},
-			expectedNeedsRemediation: false,
-			expectedNextCheck:        time.Duration(0),
-			expectedError:            true,
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			expectedNeedsRemediation:    false,
+			expectedNextCheck:           time.Duration(0),
+			expectedError:               true,
 		},
 	}
 
 	for _, tc := range testCases {
 		t.Run(tc.testCase, func(t *testing.T) {
-			needsRemediation, nextCheck, err := tc.target.needsRemediation()
+			needsRemediation, nextCheck, err := tc.target.needsRemediation(tc.timeoutForMachineToHaveNode)
 			if needsRemediation != tc.expectedNeedsRemediation {
 				t.Errorf("Case: %v. Got: %v, expected: %v", tc.testCase, needsRemediation, tc.expectedNeedsRemediation)
 			}
@@ -1704,112 +1781,6 @@ func TestNeedsRemediation(t *testing.T) {
 	}
 }
 
-func TestIsMaster(t *testing.T) {
-	testCases := []struct {
-		testCase string
-		target   *target
-		expected bool
-	}{
-		{
-			testCase: "no master",
-			target: &target{
-				Machine: mapiv1beta1.Machine{
-					TypeMeta: metav1.TypeMeta{Kind: "Machine"},
-					ObjectMeta: metav1.ObjectMeta{
-						Annotations:     make(map[string]string),
-						Name:            "test",
-						Namespace:       namespace,
-						Labels:          map[string]string{"foo": "bar"},
-						OwnerReferences: []metav1.OwnerReference{{Kind: "MachineSet"}},
-					},
-					Spec:   mapiv1beta1.MachineSpec{},
-					Status: mapiv1beta1.MachineStatus{},
-				},
-				Node: &corev1.Node{
-					ObjectMeta: metav1.ObjectMeta{
-						Name:      "test",
-						Namespace: metav1.NamespaceNone,
-						Annotations: map[string]string{
-							machineAnnotationKey: fmt.Sprintf("%s/%s", namespace, "machine"),
-						},
-						Labels: map[string]string{},
-					},
-					TypeMeta: metav1.TypeMeta{
-						Kind: "Node",
-					},
-					Status: corev1.NodeStatus{},
-				},
-				MHC: mapiv1beta1.MachineHealthCheck{},
-			},
-		},
-		{
-			testCase: "node master",
-			target: &target{
-				Machine: mapiv1beta1.Machine{
-					TypeMeta: metav1.TypeMeta{Kind: "Machine"},
-					ObjectMeta: metav1.ObjectMeta{
-						Annotations:     make(map[string]string),
-						Name:            "test",
-						Namespace:       namespace,
-						Labels:          map[string]string{"foo": "bar"},
-						OwnerReferences: []metav1.OwnerReference{{Kind: "MachineSet"}},
-					},
-					Spec:   mapiv1beta1.MachineSpec{},
-					Status: mapiv1beta1.MachineStatus{},
-				},
-				Node: &corev1.Node{
-					ObjectMeta: metav1.ObjectMeta{
-						Name:      "test",
-						Namespace: metav1.NamespaceNone,
-						Annotations: map[string]string{
-							machineAnnotationKey: fmt.Sprintf("%s/%s", namespace, "machine"),
-						},
-						Labels: map[string]string{
-							nodeMasterLabel: "",
-						},
-					},
-					TypeMeta: metav1.TypeMeta{
-						Kind: "Node",
-					},
-					Status: corev1.NodeStatus{},
-				},
-				MHC: mapiv1beta1.MachineHealthCheck{},
-			},
-			expected: true,
-		},
-		{
-			testCase: "machine master",
-			target: &target{
-				Machine: mapiv1beta1.Machine{
-					TypeMeta: metav1.TypeMeta{Kind: "Machine"},
-					ObjectMeta: metav1.ObjectMeta{
-						Annotations: make(map[string]string),
-						Name:        "test",
-						Namespace:   namespace,
-						Labels: map[string]string{
-							machineRoleLabel: machineMasterRole,
-						},
-						OwnerReferences: []metav1.OwnerReference{{Kind: "MachineSet"}},
-					},
-					Spec:   mapiv1beta1.MachineSpec{},
-					Status: mapiv1beta1.MachineStatus{},
-				},
-				Node: &corev1.Node{},
-				MHC:  mapiv1beta1.MachineHealthCheck{},
-			},
-			expected: true,
-		},
-	}
-
-	for _, tc := range testCases {
-		t.Run(tc.testCase, func(t *testing.T) {
-			if got := tc.target.isMaster(); got != tc.expected {
-				t.Errorf("Case: %v. Got: %v, expected error: %v", tc.testCase, got, tc.expected)
-			}
-		})
-	}
-}
-
 func TestMinDuration(t *testing.T) {
 	testCases := []struct {
 		testCase  string
@@ -1873,13 +1844,21 @@ func TestRemediate(t *testing.T) {
 			testCase: "no master",
 			target: &target{
 				Machine: mapiv1beta1.Machine{
-					TypeMeta: metav1.TypeMeta{Kind: "Machine"},
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "Machine",
+						APIVersion: "machine.openshift.io/v1beta1",
+					},
 					ObjectMeta: metav1.ObjectMeta{
-						Annotations:     make(map[string]string),
-						Name:            "test",
-						Namespace:       namespace,
-						Labels:          map[string]string{"foo": "bar"},
-						OwnerReferences: []metav1.OwnerReference{{Kind: "MachineSet"}},
+						Annotations: make(map[string]string),
+						Name:        "test",
+						Namespace:   namespace,
+						Labels:      map[string]string{"foo": "bar"},
+						OwnerReferences: []metav1.OwnerReference{
+							{
+								Kind:       "MachineSet",
+								Controller: pointer.BoolPtr(true),
+							},
+						},
 					},
 					Spec:   mapiv1beta1.MachineSpec{},
 					Status: mapiv1beta1.MachineStatus{},
@@ -1908,14 +1887,22 @@ func TestRemediate(t *testing.T) {
 			testCase: "node master",
 			target: &target{
 				Machine: mapiv1beta1.Machine{
-					TypeMeta: metav1.TypeMeta{Kind: "Machine"},
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "Machine",
+						APIVersion: "machine.openshift.io/v1beta1",
+					},
 					ObjectMeta: metav1.ObjectMeta{
-						Annotations:     make(map[string]string),
-						Name:            "test",
-						Namespace:       namespace,
-						Labels:          map[string]string{"foo": "bar"},
-						OwnerReferences: []metav1.OwnerReference{{Kind: "MachineSet"}},
-						UID:             "uid",
+						Annotations: make(map[string]string),
+						Name:        "test",
+						Namespace:   namespace,
+						Labels:      map[string]string{"foo": "bar"},
+						OwnerReferences: []metav1.OwnerReference{
+							{
+								Kind:       "MachineSet",
+								Controller: pointer.BoolPtr(true),
+							},
+						},
+						UID: "uid",
 					},
 					//Spec:   mapiv1beta1.MachineSpec{},
 					//Status: mapiv1beta1.MachineStatus{},
@@ -1938,15 +1925,18 @@ func TestRemediate(t *testing.T) {
 				},
 				MHC: mapiv1beta1.MachineHealthCheck{},
 			},
-			deletion:       false,
+			deletion:       true,
 			expectedError:  false,
-			expectedEvents: []string{EventSkippedMaster},
+			expectedEvents: []string{EventMachineDeleted},
 		},
 		{
 			testCase: "machine master",
 			target: &target{
 				Machine: mapiv1beta1.Machine{
-					TypeMeta: metav1.TypeMeta{Kind: "Machine"},
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "Machine",
+						APIVersion: "machine.openshift.io/v1beta1",
+					},
 					ObjectMeta: metav1.ObjectMeta{
 						Annotations: make(map[string]string),
 						Name:        "test",
@@ -1954,7 +1944,12 @@ func TestRemediate(t *testing.T) {
 						Labels: map[string]string{
 							machineRoleLabel: machineMasterRole,
 						},
-						OwnerReferences: []metav1.OwnerReference{{Kind: "MachineSet"}},
+						OwnerReferences: []metav1.OwnerReference{
+							{
+								Kind:       "MachineSet",
+								Controller: pointer.BoolPtr(true),
+							},
+						},
 					},
 					Spec:   mapiv1beta1.MachineSpec{},
 					Status: mapiv1beta1.MachineStatus{},
@@ -1962,9 +1957,9 @@ func TestRemediate(t *testing.T) {
 				Node: &corev1.Node{},
 				MHC:  mapiv1beta1.MachineHealthCheck{},
 			},
-			deletion:       false,
+			deletion:       true,
 			expectedError:  false,
-			expectedEvents: []string{EventSkippedMaster},
+			expectedEvents: []string{EventMachineDeleted},
 		},
 	}
 
@@ -1979,7 +1974,7 @@ func TestRemediate(t *testing.T) {
 			}
 			assertEvents(t, tc.testCase, tc.expectedEvents, recorder.Events)
 			machine := &mapiv1beta1.Machine{}
-			err := r.client.Get(context.TODO(), namespacedName(machine), machine)
+			err := r.client.Get(context.TODO(), namespacedName(&tc.target.Machine), machine)
 			if tc.deletion {
 				if err != nil {
 					if !errors.IsNotFound(err) {
@@ -1991,7 +1986,7 @@ func TestRemediate(t *testing.T) {
 			}
 			if !tc.deletion {
 				if !equality.Semantic.DeepEqual(*machine, tc.target.Machine) {
-					t.Errorf("Case: %v. Got: %v, expected: %v", tc.testCase, *machine, tc.target.Machine)
+					t.Errorf("Case: %v. Got: %+v, expected: %+v", tc.testCase, *machine, tc.target.Machine)
 				}
 			}
 		})
@@ -2050,12 +2045,13 @@ func TestReconcileStatus(t *testing.T) {
 func TestHealthCheckTargets(t *testing.T) {
 	now := time.Now()
 	testCases := []struct {
-		testCase               string
-		targets                []target
-		currentHealthy         int
-		needRemediationTargets []target
-		nextCheckTimesLen      int
-		errList                []error
+		testCase                    string
+		targets                     []target
+		timeoutForMachineToHaveNode time.Duration
+		currentHealthy              int
+		needRemediationTargets      []target
+		nextCheckTimesLen           int
+		errList                     []error
 	}{
 		{
 			testCase: "one healthy, one unhealthy",
@@ -2193,7 +2189,8 @@ func TestHealthCheckTargets(t *testing.T) {
 					},
 				},
 			},
-			currentHealthy: 1,
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			currentHealthy:              1,
 			needRemediationTargets: []target{
 				{
 					Machine: mapiv1beta1.Machine{
@@ -2280,7 +2277,7 @@ func TestHealthCheckTargets(t *testing.T) {
 						},
 						Spec: mapiv1beta1.MachineSpec{},
 						Status: mapiv1beta1.MachineStatus{
-							LastUpdated: &metav1.Time{Time: now.Add(time.Duration(-timeoutForMachineToHaveNode) + 1*time.Minute)},
+							LastUpdated: &metav1.Time{Time: now.Add(time.Duration(-defaultNodeStartupTimeout) + 1*time.Minute)},
 						},
 					},
 					Node: nil,
@@ -2381,16 +2378,17 @@ func TestHealthCheckTargets(t *testing.T) {
 					},
 				},
 			},
-			currentHealthy:    0,
-			nextCheckTimesLen: 2,
-			errList:           []error{},
+			timeoutForMachineToHaveNode: defaultNodeStartupTimeout,
+			currentHealthy:              0,
+			nextCheckTimesLen:           2,
+			errList:                     []error{},
 		},
 	}
 	for _, tc := range testCases {
 		recorder := record.NewFakeRecorder(2)
 		r := newFakeReconcilerWithCustomRecorder(recorder)
 		t.Run(tc.testCase, func(t *testing.T) {
-			currentHealhty, needRemediationTargets, nextCheckTimes, errList := r.healthCheckTargets(tc.targets)
+			currentHealhty, needRemediationTargets, nextCheckTimes, errList := r.healthCheckTargets(tc.targets, tc.timeoutForMachineToHaveNode)
 			if currentHealhty != tc.currentHealthy {
 				t.Errorf("Case: %v. Got: %v, expected: %v", tc.testCase, currentHealhty, tc.currentHealthy)
 			}
@@ -2598,12 +2596,59 @@ func TestIsAllowedRemediation(t *testing.T) {
 	}
 }
 
+func TestGetNodeStartupTimeout(t *testing.T) {
+	testCases := []struct {
+		name            string
+		timeout         string
+		expectError     bool
+		expectedTimeout time.Duration
+	}{
+		{
+			name:            "when the timeout is not set",
+			timeout:         "",
+			expectError:     false,
+			expectedTimeout: defaultNodeStartupTimeout,
+		},
+		{
+			name:            "when the timeout is not parsable",
+			timeout:         "a",
+			expectError:     true,
+			expectedTimeout: time.Duration(0),
+		},
+		{
+			name:            "when the timeout is  parsable",
+			timeout:         "10s",
+			expectError:     false,
+			expectedTimeout: 10 * time.Second,
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			mhc := &mapiv1beta1.MachineHealthCheck{
+				Spec: mapiv1beta1.MachineHealthCheckSpec{
+					NodeStartupTimeout: tc.timeout,
+				},
+			}
+
+			timeout, err := getNodeStartupTimeout(mhc)
+			if tc.expectError != (err != nil) {
+				t.Errorf("Case: %s. Got: %v, expected: %v", tc.name, err, tc.expectError)
+			}
+			if timeout != tc.expectedTimeout {
+				t.Errorf("Case: %s. Got: %v, expected: %v", tc.name, timeout.String(), tc.expectedTimeout.String())
+			}
+		})
+	}
+}
+
 func TestGetIntOrPercentValue(t *testing.T) {
 	int10 := intstr.FromInt(10)
 	percent20 := intstr.FromString("20%")
 	intInString30 := intstr.FromString("30")
 	invalidStringA := intstr.FromString("a")
 	invalidStringAPercent := intstr.FromString("a%")
+	invalidStringNumericPercent := intstr.FromString("1%0")
 
 	testCases := []struct {
 		name            string
@@ -2647,6 +2692,13 @@ func TestGetIntOrPercentValue(t *testing.T) {
 			expectedPercent: true,
 			expectedError:   fmt.Errorf("invalid value \"a%%\": strconv.Atoi: parsing \"a\": invalid syntax"),
 		},
+		{
+			name:            "with an '1%0' string",
+			in:              &invalidStringNumericPercent,
+			expectedValue:   0,
+			expectedPercent: false,
+			expectedError:   fmt.Errorf("invalid value \"1%%0\": strconv.Atoi: parsing \"1%%0\": invalid syntax"),
+		},
 	}
 
 	for _, tc := range testCases {
